// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.

using UnrealBuildTool;

public class Biscuit : ModuleRules
{

    // Define path for Boost library
    private string ThirdPartyPathBoostLib
    {
        get { return @"F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/BoostAsio/Windows/stage/lib"; }
        //$$$$ get { return @"..\..\SmartFoxClientAPI\Lib"; }
    }

    // Define path for Boost includes
    private string ThirdPartyPathBoostInclude
    {
        get { return @"F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/BoostAsio/Windows"; }
        //$$$$ get { return @".\SmartFoxClientAPI\Include\Core\BoostAsio\Windows"; }
    }

    // Define path for SmartFoxClientAPI library
    private string ThirdPartyPathSmartFoxClientAPI
    {
        get { return @"F:/FlyingCookie/Biscuit/SmartFoxClientAPI/SmartFoxClientApi/VS2015/x64/Release Static Library"; }
        //$$$$ get { return @"..\..\SmartFoxClientAPI\Lib"; }
    }

    // Define path for SmartFoxClientAPI includes
    private string ThirdPartyPathSmartFoxClientAPIInclude
    {
        get { return @"F:/FlyingCookie/Biscuit/SmartFoxClientAPI"; }
        //$$$$ get { return @".\SmartFoxClientAPI\Include"; }
    }


    // Define path for SmartFoxClientAPI library
    private string ThirdPartyOpenSSLLib
    {
        get { return @"F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/OpenSSL/Win/lib/Win64/VS2015"; }
        //$$$$ get { return @"..\..\SmartFoxClientAPI\Lib"; }
    }

    // Define path for SmartFoxClientAPI includes
    private string ThirdPartyOpenSSLInclude
    {
        get { return @"F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/OpenSSL/Win/include/openssl"; }
        //$$$$ get { return @".\SmartFoxClientAPI\Include"; }
    }

    //private string ThirdPartyZLibReleaseT
    //{
    //    get { return @"C:/FlyingCookie/Biscuit/zlib-1.2.11/"; }
    //    //$$$$ get { return @"..\..\SmartFoxClientAPI\Lib"; }
    //}

    //// Define path for SmartFoxClientAPI library
    //private string ThirdPartyZIncludeReleaseT
    //{
    //    get { return @"C:/FlyingCookie/Biscuit/zlib-1.2.11"; }
    //    //$$$$ get { return @"..\..\SmartFoxClientAPI\Lib"; }
    //}

    /// <summary>
    /// Load Boost library
    /// </summary>
    /// <param name="Target">Target environment</param>
    /// <returns>True on success</returns>
    public bool LoadBoostLib(ReadOnlyTargetRules Target)
    {
        bool isLibrarySupported = false;

        if ((Target.Platform == UnrealTargetPlatform.Win64) || (Target.Platform == UnrealTargetPlatform.Win32))
        {
            isLibrarySupported = true;

            string PlatformString = (Target.Platform == UnrealTargetPlatform.Win64) ? "x64" : "x86";

            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyPathBoostLib, "libboost_system-vc140-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyPathBoostLib, "libboost_chrono-vc140-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyPathBoostLib, "libboost_date_time-vc140-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyPathBoostLib, "libboost_regex-vc140-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyPathBoostLib, "libboost_thread-vc140-mt-1_64.lib"));
            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyPathBoostLib, "libboost_filesystem-vc140-mt-1_64.lib"));
        }

        if (isLibrarySupported)
        {
            PublicIncludePaths.Add(ThirdPartyPathBoostInclude);
        }

        Definitions.Add(string.Format("WITH_BOOST_BINDING={0}", isLibrarySupported ? 1 : 0));

        return isLibrarySupported;
    }

    /// <summary>
    /// Load SmartFoxServer Client API library
    /// </summary>
    /// <param name="Target">Target environment</param>
    /// <returns>True on success</returns>
    public bool LoadSmartFoxClientAPILib(ReadOnlyTargetRules Target)
    {
        bool isLibrarySupported = false;

        if ((Target.Platform == UnrealTargetPlatform.Win64) || (Target.Platform == UnrealTargetPlatform.Win32))
        {
            isLibrarySupported = true;

            string PlatformString = (Target.Platform == UnrealTargetPlatform.Win64) ? "x64" : "x86";


            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyPathSmartFoxClientAPI, "SmartFoxClientApi.lib"));

        }

        if (isLibrarySupported)
        {
            // Include path
            PublicIncludePaths.Add(ThirdPartyPathSmartFoxClientAPIInclude);
        }

        Definitions.Add(string.Format("WITH_SMARTFOX_BINDING={0}", isLibrarySupported ? 1 : 0));

        return isLibrarySupported;
    }

    public bool LoadOpenSSLLib(ReadOnlyTargetRules Target)
    {
        bool isLibrarySupported = false;

        if ((Target.Platform == UnrealTargetPlatform.Win64) || (Target.Platform == UnrealTargetPlatform.Win32))
        {
            isLibrarySupported = true;

            string PlatformString = (Target.Platform == UnrealTargetPlatform.Win64) ? "x64" : "x86";

            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyOpenSSLLib, "libeay32.lib"));
            PublicAdditionalLibraries.Add(System.IO.Path.Combine(ThirdPartyOpenSSLLib, "ssleay32.lib"));
        }

        if (isLibrarySupported)
        {
            // Include path
            PublicIncludePaths.Add(ThirdPartyOpenSSLInclude);
        }

        Definitions.Add(string.Format("WITH_OPENSSL_BINDING={0}", isLibrarySupported ? 1 : 0));

        return isLibrarySupported;
    }

    //public bool loadzlib(readonlytargetrules target)
    //{
    //    bool islibrarysupported = false;

    //    if ((target.platform == unrealtargetplatform.win64) || (target.platform == unrealtargetplatform.win32))
    //    {
    //        islibrarysupported = true;

    //        string platformstring = (target.platform == unrealtargetplatform.win64) ? "x64" : "x86";

    //        publicadditionallibraries.add(system.io.path.combine(thirdpartyzlibrelease, "zlibwapi.lib"));
    //        publicadditionallibraries.add(system.io.path.combine(thirdpartyzlibreleaset, "zlibstat.lib"));
    //    }

    //    if (islibrarysupported)
    //    {
    //        include path
    //        publicincludepaths.add(thirdpartyzincludereleaset);
    //    }

    //    definitions.add(string.format("with_zlib2_binding={0}", islibrarysupported ? 1 : 0));

    //    return islibrarysupported;
    //}

    public Biscuit(ReadOnlyTargetRules Target) : base(Target)
    {
        PCHUsage = PCHUsageMode.UseExplicitOrSharedPCHs;

        PublicDependencyModuleNames.AddRange(new string[] { "Core", "CoreUObject", "Engine", "InputCore", "HeadMountedDisplay", "UMG", "SlateCore", "Slate", "RHI" });

        // Load Boost library
        LoadBoostLib(Target);

        // Load SmartFoxClientAPI library
        LoadSmartFoxClientAPILib(Target);

        LoadOpenSSLLib(Target);

        //LoadZLib(Target);

        // Enable RTTI
        bUseRTTI = true;

    }
}