// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.

#pragma once
// -------------------------------------------------------------------------
// API defines
// -------------------------------------------------------------------------
#define SMARTFOX_MAXIMUMWAITFORCONNECTIONESTABLISHMENT	10000					// UM: milliseconds. The time to wait the establishment of connection with SmartFox

// -------------------------------------------------------------------------
// API includes
// -------------------------------------------------------------------------
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/SmartFox.h"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/BaseEvent.h"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Requests/JoinRoomRequest.h"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Requests/PublicMessageRequest.h"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Bitswarm/BitSwarmEvent.h"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Util/EventDispatcher.h"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Requests/LoginRequest.h"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Util/SFSErrorCodes.h"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/BoostAsio/Windows/boost/shared_ptr.hpp"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/BoostAsio/Windows/boost/filesystem.hpp"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/BoostAsio/Windows/boost/chrono.hpp"
#include "F:/FlyingCookie/Biscuit/SmartFoxClientAPI/Core/BoostAsio/Windows/boost/regex.hpp"
#include <iostream>

#include "CoreMinimal.h"
#include "SfsChatWidget.h"
#include "GameFramework/Character.h"
#include "BiscuitCharacter.generated.h"

using namespace Sfs2X;

UCLASS(Blueprintable)
class ABiscuitCharacter : public ACharacter
{
	GENERATED_BODY()

public:
	ABiscuitCharacter();

	//
	virtual void BeginPlay() override;

	// Called every frame.
	virtual void Tick(float DeltaSeconds) override;

	/** Returns TopDownCameraComponent subobject **/
	FORCEINLINE class UCameraComponent* GetTopDownCameraComponent() const { return TopDownCameraComponent; }
	/** Returns CameraBoom subobject **/
	FORCEINLINE class USpringArmComponent* GetCameraBoom() const { return CameraBoom; }
	/** Returns CursorToWorld subobject **/
	FORCEINLINE class UDecalComponent* GetCursorToWorld() { return CursorToWorld; }

private:
	/** Top down camera */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = Camera, meta = (AllowPrivateAccess = "true"))
	class UCameraComponent* TopDownCameraComponent;

	/** Camera boom positioning the camera above the character */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = Camera, meta = (AllowPrivateAccess = "true"))
	class USpringArmComponent* CameraBoom;

	/** A decal that projects to the cursor location. */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = Camera, meta = (AllowPrivateAccess = "true"))
	class UDecalComponent* CursorToWorld;

	// --------------------------
	// Smartfox 
	// --------------------------

public:
	// The class that will be used for the players Inventory UI
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "Player, HUD and UI")
		TSubclassOf<class USfsChatWidget> SfsChatUIClass;

	// The instance of the players Inventory UI Widget
	UPROPERTY(EditDefaultsOnly, BlueprintReadOnly, Category = "Player, HUD and UI")
		class USfsChatWidget* SfsChatWidget;

private:

	/** Pointer to SmartFox. Note: IT MUST BE A SHARED_PTR ! */
	boost::shared_ptr<Sfs2X::SmartFox> m_ptrSmartFox;

	/** Event to notify the establishment of the connection with SmartFox */
	HANDLE m_hSmartFoxConnectionEstablished;

	/** Flag to preserve the status of connection with SmartFox Server (true when connection is established) */
	bool m_bSmartFoxConnectionEstablished;

	/** Pointer to SmartFox login request */
	boost::shared_ptr<IRequest> m_ptrSmartFoxLoginRequest;

	/** Pointer to SmartFox join room request */
	boost::shared_ptr<IRequest> m_ptrSmartFoxJoinRoomRequest;

	/** Pointer to SmartFox public message request */
	boost::shared_ptr<IRequest> m_ptrSmartFoxPublicMessageRequest;

private:

	/**
	* Listener to handle the SmartFox connection event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxConnection(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox connection lost event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxConnectionLost(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox connection retry event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxConnectionRetry(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox connection resume event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxConnectionResume(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox disconnection event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxDisconnection(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox login event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxLogin(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox login error event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxLoginError(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox logout event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxLogout(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox join room event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxRoomJoin(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox join room error event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxRoomJoinError(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

	/**
	* Listener to handle the SmartFox message receive event
	* @param ptrContext		custom context
	* @param ptrEvent		connection event
	*/
	static void OnSmartFoxPublicMessage(unsigned long long ptrContext, boost::shared_ptr<BaseEvent> ptrEvent);

public:

	/**
	* Initialize the binding with Smart Fox server
	*/
	void SmartFoxInitialize();

	/**
	* SmartFox login
	* @param username		user name
	*/
	void SmartFoxLogin(string username);

	/**
	* SmartFox join room
	* @param roomName		room name
	*/
	void SmartFoxJoinRoom(string roomName);

	/**
	* SmartFox chat send message
	* @param username		message to send
	*/
	void SmartFoxSendChatMessage(string message);

	/**
	* Get the connection status with SmartFox Server
	* @return true when connected
	*/
	bool IsSmartFoxServerConnected();
};

